import os

import pytest
import pandas as pd

from semeio.jobs.rft.gendata_rft import _write_simdata, _write_gen_data_files


@pytest.mark.parametrize(
    "dataname, input_data, expected_result",
    [
        ("pressure", {"order": [1, 2], "pressure": [10.0, 20.0]}, ["10.0\n", "20.0\n"]),
        ("pressure", {"order": [1, 2]}, ["-1\n", "-1\n"]),
        ("sgas", {"order": [1, 2], "sgas": [0.1, 0.2]}, ["0.1\n", "0.2\n"]),
        ("sgas", {"order": [2, 1], "sgas": [0.2, 0.1]}, ["0.1\n", "0.2\n"]),
    ],
)
def test_write_simdata(tmpdir, dataname, input_data, expected_result):
    with tmpdir.as_cwd():
        df = pd.DataFrame(input_data)
        _write_simdata("some_file_name", dataname, df)

        with open("some_file_name", "r") as fin:
            result = fin.readlines()
        assert result == expected_result


def test_write_gen_data_files_always_pressure(tmpdir):
    """Check that a file with pressure is always written, even it if is not
    in the dataframe.

    Saturation data is only outputted when present (might change in the future).
    """
    tmpdir.chdir()
    dframe = pd.DataFrame({"order": [1, 2], "is_active": [0, 0]})
    _write_gen_data_files(dframe, ".", "A-1", 0)
    pressure_file = "RFT_A-1_0"
    assert os.path.exists(pressure_file)
    with open(pressure_file) as f_handle:
        pressure_lines = f_handle.read().splitlines()
    assert pressure_lines == ["-1", "-1"]

    swat_file = "RFT_A-1_SWAT_0"
    assert not os.path.exists(swat_file)



from semeio.workflows.correlated_observations_scaling.scaled_matrix import DataMatrix


class ObservationScaleFactor(object):
    def __init__(
        self,
        reporter,
        measured_data,
    ):
        """Creates a ScalingJob instance with the given obs_keys, obs,
        obs_with_data and user_config_dict."""
        self._reporter = reporter
        self._measured_data = measured_data

    def perform_pca(self, threshold):
        matrix = DataMatrix(self._measured_data.data)
        matrix.normalize_by_std()

        nr_components, singular_values = matrix.get_nr_primary_components(
            threshold=threshold
        )
        self._reporter.publish("svd", list(singular_values))
        return nr_components, singular_values

    def get_scaling_factor(self, threshold):
        """
        Collects data performs pca, and returns scaling factor, assumes validated input.
        """
        nr_observations = self._measured_data.data.shape[1]
        nr_components, singular_values = self.perform_pca(threshold)
        scale_factor = DataMatrix.get_scaling_factor(nr_observations, nr_components)

        self._reporter.publish("scale_factor", scale_factor)

        return scale_factor
